"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var http_ios_1 = require("../http.ios");
var types = require("@nativescript/core/utils/types");
var connections = new Array();
var delegates = new Array();
var WebsocketConnection = (function () {
    function WebsocketConnection(nativeConnection) {
        this.nativeConnection = nativeConnection;
    }
    WebsocketConnection.prototype.queueSize = function () {
        return 0;
    };
    WebsocketConnection.prototype.send = function (text) {
        this.nativeConnection.send(text);
    };
    WebsocketConnection.prototype.sendBinary = function (bytes) {
        var buffer = bytes;
        this.nativeConnection.send(NSData.dataWithData(buffer));
    };
    WebsocketConnection.prototype.close = function (code, reason) {
        return this.nativeConnection.closeWithCodeReason(code, reason);
    };
    WebsocketConnection.prototype.cancel = function () {
        this.nativeConnection.close();
    };
    return WebsocketConnection;
}());
exports.WebsocketConnection = WebsocketConnection;
var SRWebSocketDelegateImpl = (function (_super) {
    __extends(SRWebSocketDelegateImpl, _super);
    function SRWebSocketDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SRWebSocketDelegateImpl.prototype.webSocketDidCloseWithCodeReasonWasClean = function (webSocket, code, reason, wasClean) {
        this.callbacks.onClosed(code, reason);
    };
    SRWebSocketDelegateImpl.prototype.webSocketDidFailWithError = function (webSocket, error) {
        this.callbacks.onFailure(error.localizedDescription);
    };
    SRWebSocketDelegateImpl.prototype.webSocketDidOpen = function (webSocket) {
        this.callbacks.onOpen();
    };
    SRWebSocketDelegateImpl.prototype.webSocketDidReceiveMessage = function (webSocket, message) {
        if (message && message.bytes) {
            this.callbacks.onBinaryMessage(interop.bufferFromData(message));
        }
        else {
            this.callbacks.onMessage(message);
        }
    };
    SRWebSocketDelegateImpl.ObjCProtocols = [SRWebSocketDelegate];
    return SRWebSocketDelegateImpl;
}(NSObject));
function newWebsocketConnection(options, callbacks) {
    return new Promise(function (resolve, reject) {
        var urlRequest = NSMutableURLRequest.requestWithURL(NSURL.URLWithString(options.url));
        var userAgent;
        if (options.headers) {
            for (var key in options.headers) {
                if (key.toLowerCase() === "user-agent") {
                    userAgent = options.headers[key];
                }
            }
        }
        if (!userAgent) {
            if (!options.headers) {
                options.headers = {};
            }
            options.headers[http_ios_1.USER_AGENT_HEADER] = http_ios_1.getCurrentUserAgent() ? http_ios_1.getCurrentUserAgent() : http_ios_1.USER_AGENT;
        }
        if (options.headers) {
            for (var header in options.headers) {
                urlRequest.setValueForHTTPHeaderField(options.headers[header] + "", header);
            }
        }
        if (types.isNumber(options.timeout)) {
            urlRequest.timeoutInterval = options.timeout / 1000;
        }
        var delegate = SRWebSocketDelegateImpl.new();
        delegate.callbacks = callbacks;
        var socket = SRWebSocket.alloc().initWithURLRequest(urlRequest);
        socket.delegate = delegate;
        socket.open();
        connections.push(socket);
        delegates.push(delegate);
        var websocketConnection = new WebsocketConnection(socket);
        resolve(websocketConnection);
    });
}
exports.newWebsocketConnection = newWebsocketConnection;
//# sourceMappingURL=index.ios.js.map